/*
	des.h
	Copyright (C) 2001 David Fifield

	This file is part of descrypt.

	descrypt is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2, or (at your option)
	any later version.

	descrypt is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef _DES_H
#define _DES_H

#include <stddef.h>

enum {
	NUM_ROUNDS = 16
};

enum {
	BLOCK_SIZE = 8,
	BLOCK_BITS = BLOCK_SIZE * 8,
	SUBKEY_SIZE = 6,
	SUBKEY_BITS = SUBKEY_SIZE * 8,
	KEY_SIZE = 7,
	KEY_BITS = KEY_SIZE * 8,
	LONG_KEY_SIZE = 8,
	LONG_KEY_BITS = LONG_KEY_SIZE * 8
};

enum {
	DES_ENCRYPT = 0, DES_DECRYPT
};

void des_crypt(unsigned char to[BLOCK_SIZE],
               const unsigned char from[BLOCK_SIZE],
               unsigned char subkey[NUM_ROUNDS][SUBKEY_SIZE]);

void des_round(unsigned char left[BLOCK_SIZE / 2],
               unsigned char right[BLOCK_SIZE / 2],
               unsigned char subkey[SUBKEY_SIZE]);

void des_key_init(unsigned char subkey[NUM_ROUNDS][SUBKEY_SIZE],
                  unsigned char long_key[LONG_KEY_SIZE], int crypt_dir);

int des_key_parity(const unsigned char long_key[LONG_KEY_SIZE]);

void bit_permute(unsigned char *to, const unsigned char *from,
                 const unsigned char *perm, unsigned int num);

void init_perm(unsigned char to[BLOCK_SIZE],
               const unsigned char from[BLOCK_SIZE]);

void final_perm(unsigned char to[BLOCK_SIZE],
                const unsigned char from[BLOCK_SIZE]);

void exp_perm(unsigned char to[SUBKEY_SIZE],
              const unsigned char from[BLOCK_SIZE / 2]);

void s_box_sub(unsigned char to[BLOCK_SIZE / 2],
               unsigned char from[SUBKEY_SIZE]);

void key_shift_left(unsigned char key[KEY_SIZE], unsigned int num);

void key_shift_right(unsigned char key[KEY_SIZE], unsigned int num);

void xor(void *s, const void *t, size_t num);

#endif

